# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class SubscriptionCollection(RedfishJsonValidator):
    """Creates a Subscription Collection Redfish dict

        Populates self.redfish with some hardcoded SubscriptionCollection
        values
    """

    SCHEMA_NAME = 'EventDestinationCollection'

    def __init__(self, all_subscriptions):
        """SubscriptionCollection constructor.

            Populates self.redfish with hardcoded SubscriptionCollection
            values.

            all_subscriptions: Dictionary with all subscriptions.
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Event Subscriptions Collection"
        self.redfish["Members@odata.count"] = len(all_subscriptions)
        self.redfish["Members"] = list()

        for subscription_id in all_subscriptions:
            member = collections.OrderedDict()
            member["@odata.id"] = \
                "/redfish/v1/EventService" \
                "/EventSubscriptions/{}".format(
                    all_subscriptions[subscription_id].get_id())

            self.redfish["Members"].append(member)

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#EventDestinationCollection" \
            ".EventDestinationCollection"
        self.redfish["@odata.id"] = \
            "/redfish/v1/EventService/EventSubscriptions/"

        self._validate()



#!./redfish-venv/bin/python
# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import re
from xml.dom import minidom
import xml.etree.ElementTree as ET

# Project libs
from oneview_redfish_toolkit import config


class Metadata(object):
    """Creates a redfish $metadata XML

    """

    def __init__(self, reference_list):
        """Constructor

            Populates self.redfish with a hardcoded odata values
        """

        metadata = ET.Element("edmx:Edmx")
        metadata.set(
            "xmlns:edmx", "http://docs.oasis-open.org/odata/ns/edmx")
        metadata.set("Version", "4.0")
        for key in reference_list:
            reference = ET.SubElement(metadata, "edmx:Reference")
            reference.set(
                "Uri",
                "http://redfish.dmtf.org/schemas/v1/{}_v1.xml".format(key))
            include = ET.SubElement(reference, "edmx:Include")
            include.set("Namespace", key)
            if key.find('Collection') == -1:
                include2 = ET.SubElement(reference, "edmx:Include")
                file_version = re.search(
                    '.+\.v(\d+_\d+_\d+)\.json', reference_list[key]).group(1)
                file_version = file_version.replace("_", ".")
                include2.set("Namespace", key + "." + file_version)
        reference = ET.SubElement(metadata, "edmx:Reference")
        reference.set(
            "Uri",
            "http://redfish.dmtf.org/schemas/v1/RedfishExtensions_v1.xml")
        include = ET.SubElement(reference, "edmx:Include")
        include.set("Namespace", "RedfishExtensions.1.0.0")
        include.set("Alias", "RedfishExtensions")
        data_services = ET.SubElement(metadata, "edmx:DataServices")
        schema = ET.SubElement(data_services, "Schema")
        schema.set("NameSpace", "Service")
        entity_container = ET.SubElement(schema, "EntityContainer")
        entity_container.set("Name", "Service")
        entity_container.set("Extends", "ServiceRoot.1.0.0.ServiceContainer")

        self.metadata = metadata

    def serialize(self):
        xml_str = ET.tostring(self.metadata, encoding="UTF-8").decode("UTF-8")
        # If prettify
        if config.get_config()['redfish']['xml_prettify'] == "True":
            return minidom.parseString(xml_str).toprettyxml(
                encoding="UTF-8", indent="    ").decode("UTF-8")
        else:
            return xml_str



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import time

from http.client import HTTPConnection
from threading import Thread
from urllib.parse import urlparse


class EventDispatcher(Thread):
    """Dispatches an event to its subscribers"""

    RESPONSE_HEADER = {'Content-Type': 'application/json'}

    def __init__(self, event, subscription, retry_attempts, retry_interval):
        """EventDispatcher constructor

            Dispatches an event to its subscribers.

            Args:
                event: The event object to be dispatched
                subscription: Subscriber information
                retry_attempts: Number of attempts to dispatch the event
                retry_interval: Number in seconds of the interval between
                each retry attempt
        """
        Thread.__init__(self)

        self.event = event
        self.subscription = subscription
        self.retry_attempts = retry_attempts
        self.retry_interval = retry_interval

    def run(self):
        attempts_counter = 0
        done = False

        try:
            url = urlparse(self.subscription.redfish['Destination'])
            json_str = self.event.serialize()

            while (attempts_counter < self.retry_attempts and not done):
                attempts_counter += 1

                try:
                    connection = HTTPConnection(url.hostname, port=url.port)
                    connection.request(
                        'POST', url.path, json_str, self.RESPONSE_HEADER)

                    done = True
                except Exception as e:
                    logging.exception(
                        'Could not dispatch event to {}. '
                        'Error: {}'.format(url.netloc, e))

                    time.sleep(self.retry_interval)
                finally:
                    connection.close()
        except Exception as e:
            logging.exception(
                'Error getting event and/or subscriber information: {}'
                .format(e))



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# THIS FILE IS MANAGED BY THE GLOBAL REQUIREMENTS REPO - DO NOT EDIT
import setuptools

# In python < 2.7.4, a lazy loading of package `pbr` will break
# setuptools if some other modules registered functions in `atexit`.
# solution from: http://bugs.python.org/issue15881#msg170215
try:
    import multiprocessing  # noqa
except ImportError:
    pass

setuptools.setup(
    packages=setuptools.find_packages(),
    scripts=['scripts/oneview-redfish-toolkit'],
    setup_requires=["pbr"],
    download_url="https://github.com/HewlettPackard/oneview-redfish-toolkit/"
    "tarball/0.4.0",
    python_requires='>=3.5',
    pbr=True
    )



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from oneview_redfish_toolkit.api.chassis import Chassis
from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.util.power_option import \
    RESET_ALLOWABLE_VALUES_LIST


class BladeChassis(Chassis):
    """Creates a Blade Chassis Redfish dict

         Populates self.redfish with some hardcoded Chassis values and
         with the response of OneView server hardware resources.
    """

    def __init__(self, server_hardware, manager_uuid):
        """BladeChassis constructor

        Populates self.redfish with hardcoded Chassis values
        and with the response of OneView server hardware.

        Args:
            server_hardware: An object containing hardware to
                      create the Redfish JSON.
            manager_uuid: Oneview's current manager uuid.
        """

        super().__init__(server_hardware)

        self.redfish["ChassisType"] = "Blade"
        self.redfish["Model"] = server_hardware["model"]
        self.redfish["IndicatorLED"] = self. \
            _map_indicator_led(server_hardware["uidState"])

        self.redfish["Links"]["ComputerSystems"] = list()

        server_profile_uri = server_hardware["serverProfileUri"]
        if server_profile_uri:
            system_id = server_profile_uri.split("/")[-1]
            self.redfish["Links"]["ComputerSystems"].append(
                {
                    "@odata.id": ComputerSystem.BASE_URI + "/" + system_id
                }
            )

        self.redfish["Links"]["ManagedBy"] = list()
        if manager_uuid:
            self.redfish["Links"]["ManagedBy"].append(
                collections.OrderedDict())
            self.redfish["Links"]["ManagedBy"][0]["@odata.id"] = \
                "/redfish/v1/Managers/" + manager_uuid

        if server_hardware["locationUri"] is not None:
            self.redfish["Links"]["ContainedBy"] = collections.OrderedDict()
            self.redfish["Links"]["ContainedBy"]["@odata.id"] = \
                "/redfish/v1/Chassis/" \
                + server_hardware["locationUri"].split("/")[-1]

        self.redfish["NetworkAdapters"] = dict()
        self.redfish["NetworkAdapters"]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware["uuid"] + \
            "/NetworkAdapters/"

        self.redfish["Actions"] = collections.OrderedDict()
        self.redfish["Actions"]["#Chassis.Reset"] = \
            collections.OrderedDict()
        self.redfish["Actions"]["#Chassis.Reset"]["target"] = \
            "/redfish/v1/Chassis" + "/" + \
            server_hardware["uuid"] + \
            "/Actions/Chassis.Reset"
        self.redfish["Actions"]["#Chassis.Reset"][
            "ResetType@Redfish.AllowableValues"] = \
            RESET_ALLOWABLE_VALUES_LIST

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class ResourceBlockCollection(RedfishJsonValidator):
    """Creates a ResourceBlock Collection Redfish dict

        Populates self.redfish with some hardcoded ResourceBlockCollection
        values and with the response of Oneview.
    """

    SCHEMA_NAME = 'ResourceBlockCollection'
    BASE_URI = '/redfish/v1/CompositionService/ResourceBlocks'

    def __init__(self,
                 server_hardware=[],
                 server_profile_templates=[],
                 drives=[],
                 external_volume_list=[]):
        """ResourceBlockCollection constructor

            Populates self.redfish with a hardcoded ResourceBlockCollection
            values and with the response of Oneview.
        """

        super().__init__(self.SCHEMA_NAME)

        self.members = server_hardware \
            + server_profile_templates \
            + drives \
            + external_volume_list

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Resource Block Collection"

        self.redfish["Members"] = list()
        self._fill_resource_block_list()
        self.redfish["Members@odata.count"] = len(self.redfish["Members"])

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ResourceBlockCollection" \
            ".ResourceBlockCollection"
        self.redfish["@odata.id"] = self.BASE_URI

        self._validate()

    def _fill_resource_block_list(self):
        """Mounts the list of the ResourceBlock members

            Populates self.redfish["Members"] with the links to ResourceBlocks.
        """

        for block in self.members:
            block_id = block.get("uuid", block["uri"].split("/")[-1])

            resource_block = dict()

            resource_block["@odata.id"] = self.BASE_URI + "/" + block_id

            self.redfish["Members"].append(resource_block)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import threading


lock = threading.Lock()

map_category_resources_ov = None


class CategoryResource(object):
    def __init__(self, resource_id, resource, function):
        self.resource_id = resource_id
        self.resource = resource
        self.function = function


def init_map_category_resources():
    global map_category_resources_ov
    map_category_resources_ov = dict()


def _get_map_category_resources():
    return map_category_resources_ov


def set_map_category_resources_entry(resource_id, resource, function):
    # Check if the resource's category is already cached
    if get_category_by_resource_id(resource_id):
        return

    with lock:
        global map_category_resources_ov
        map_category_resources_ov[resource_id] = \
            CategoryResource(resource_id, resource, function)


def get_category_by_resource_id(resource_id):
    """Get cached resource category by resource ID"""
    cached_category = _get_map_category_resources().get(resource_id)

    return cached_category



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import g

# Modules own libs
from oneview_redfish_toolkit.config import PERFORMANCE_LOGGER_NAME


def single_oneview(original_func):
    """Python decorator for endpoints with single OneView context"""
    def new_function(*args, **kwargs):
        """Function to set single OneView context"""
        logging.getLogger(PERFORMANCE_LOGGER_NAME).debug(
            "in Single OneView Context")
        set_single_oneview_context()
        return original_func(*args, **kwargs)

    new_function.__name__ = original_func.__name__
    return new_function


def set_single_oneview_context():
    """Set to use the same OneView IP in the same request"""
    g.single_oneview_context = True


def is_single_oneview_context():
    """Check if it ot be used the same OneView IP on the request context"""
    return 'single_oneview_context' in g


def get_single_oneview_ip():
    """Get the same OneView's IP in request context"""
    if 'single_oneview_ip' in g:
        return g.single_oneview_ip
    return None


def set_single_oneview_ip(oneview_ip):
    """Set OneView's IP to be used in the same request context"""
    if not get_single_oneview_ip():
        g.single_oneview_ip = oneview_ip



# -*- coding: utf-8 -*-

# Copyright (2017) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# 3rd party libs
from flask import Blueprint

# own libs
from oneview_redfish_toolkit.api.manager_collection import ManagerCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit import multiple_oneview

manager_collection = Blueprint("manager_collection", __name__)


@manager_collection.route("/redfish/v1/Managers/", methods=["GET"])
def get_manager_collection():
    """Get the Redfish Manager Collection.

        Return ManagerCollection redfish JSON.
        Logs exception of any error and return
        Internal Server Error or Not Found.

        Returns:
            JSON: Redfish json with ManagerCollection.

        Exceptions:
            Exception: Generic error, logs the exception and call abort(500).
    """

    oneview_appliances = multiple_oneview.get_map_appliances()
    mc = ManagerCollection(oneview_appliances)

    return ResponseBuilder.success(mc)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class ZoneCollection(RedfishJsonValidator):
    """Creates a Zone Collection Redfish dict

        Populates self.redfish with some hardcoded ZoneCollection
        values and with the response of OneView.
    """

    SCHEMA_NAME = 'ZoneCollection'
    BASE_URI = '/redfish/v1/CompositionService/ResourceZones'

    def __init__(self, zone_ids):
        """ZoneCollection constructor

            Populates self.redfish with some hardcoded ZoneCollection
            values and with the response of OneView.

            Args:
                zone_ids: Zone ids based on Oneview data
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Resource Zone Collection"

        self.redfish["Members@odata.count"] = len(zone_ids)
        self.redfish["Members"] = list()
        self._fill_redfish_members(zone_ids)

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ZoneCollection.ZoneCollection"
        self.redfish["@odata.id"] = \
            "/redfish/v1/CompositionService/ResourceZones"

        self._validate()

    def _fill_redfish_members(self, zone_ids):
        """Mounts the list of Redfish members

            Populates self.redfish["Members"] with the links
            to Redfish Resource Zones.
        """
        for zone_id in zone_ids:
            resource_zone = {
                "@odata.id": self.BASE_URI + "/" + zone_id
            }
            self.redfish["Members"].append(resource_zone)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import collections
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import Response
from flask_api import status

# own libs
from oneview_redfish_toolkit.api.metadata import Metadata
from oneview_redfish_toolkit.api import schemas

metadata = Blueprint('metadata', __name__)


@metadata.route('/redfish/v1/$metadata', methods=["GET"])
def get_metadata():
    """Gets metadata

        Generates the XML with schemas used by this system
    """

    try:
        schemas_dict = collections.OrderedDict(schemas.SCHEMAS)
        mtdt = Metadata(schemas_dict)
        json_str = mtdt.serialize()
        return Response(
            response=json_str,
            status=200,
            mimetype='text/xml')
    except Exception as e:
        logging.exception('metadata error: {}'.format(e))
        abort(status.HTTP_500_INTERNAL_SERVER_ERROR)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import inspect
import json
import logging
import os
import uuid

from filelock import FileLock
from flask import abort
from flask import Blueprint
from flask import request
from flask import Response
from flask_api import status
from jsonschema.exceptions import ValidationError
import validators

from oneview_redfish_toolkit.api import scmb
from oneview_redfish_toolkit.api.subscription import Subscription
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import util


subscription = Blueprint("subscription", __name__)
REDFISH_TOOLKIT_BASE_DIR = 'oneview_redfish_toolkit'
ALL_SUBSCRIPTION_FILE = 'all_subscription.json'


def _all_subscription_file():
    base_dir = os.path.abspath(os.path.join(
        os.path.dirname(inspect.getfile(inspect.currentframe())), '..'))
    logging.info(base_dir)
    return os.path.join(base_dir, ALL_SUBSCRIPTION_FILE)


def get_file_content():
    file_content = None
    try:
        with open(_all_subscription_file()) as f:
            file_content = json.load(f)
    except Exception as e:
        logging.exception("Error while reading File: " + str(e))

    return file_content


def _get_file_lock():
    lock_path = _all_subscription_file() + '.lock'
    lock = FileLock(lock_path)
    return lock


def _update_all_subscription(file_content):
    try:
        with open(_all_subscription_file(), 'w+') as f:
            f.write(json.dumps(file_content, indent=4))
    except Exception as e:
        raise e


def _add_subscription_to_file(subscription):
    file_content = get_file_content()
    if file_content:
        if not file_content.get('members'):
            # On first subscription request, start SCMB service
            if config.auth_mode_is_session():
                token = request.headers.get('x-auth-token')
                scmb.init_event_service(token)
            else:
                scmb.init_event_service()
        file_content['members'].append(subscription)
        _update_all_subscription(file_content)


def _delete_subscription_from_file(subscription_id):
    file_content = get_file_content()
    if file_content and file_content.get('members'):
        sc = [x for x in file_content.get('members')
              if x['Id'] == subscription_id]
        file_content["members"].remove(sc[0])
        _update_all_subscription(file_content)


def _is_duplicate_subscription(destination):
    file_content = get_file_content()
    if file_content and file_content.get('members'):
        for i in file_content.get('members'):
            if i.get('Destination') == destination:
                return True
    return False


@subscription.route(
    "/redfish/v1/EventService/EventSubscriptions/", methods=["POST"])
def add_subscription():
    """Add the Redfish Subscription.

        Add a new subscription when this POST operation is requested.
        The body of the request must have the Destination,
        an array of EventTypes. Context is optional.

        EventTypes:
            - ResourceUpdated
            - ResourceAdded
            - ResourceRemoved
            - Alert

        Returns:
            JSON: JSON with Subscription information.

        Exception:
            KeyError: When occur a key mapping error.
            return abort(400)

    """

    # get Destination, EventTypes and Context from post
    # generate subscription uuid
    # add subscription in subscriptions_by_type
    # add subscription in all_subscriptions

    try:
        body = request.get_json()
        destination = body["Destination"]

        if not validators.url(destination):
            abort(status.HTTP_400_BAD_REQUEST,
                  "Destination must be an URI.")

        # read all_subscription file and check if destination already present
        is_duplicate = _is_duplicate_subscription(destination)
        if is_duplicate:
            abort(status.HTTP_400_BAD_REQUEST, "Destination is duplicate")

        event_types = body["EventTypes"]

        if not event_types:
            abort(status.HTTP_400_BAD_REQUEST,
                  "EventTypes cannot be empty.")

        context = body.get("Context")
    except KeyError:
        error_message = "Invalid JSON key. The JSON request body " \
                        "must have the keys Destination and EventTypes. " \
                        "The Context is optional."
        abort(status.HTTP_400_BAD_REQUEST, error_message)

    subscription_id = str(uuid.uuid1())

    try:
        # Build Subscription object and validates it
        sc = Subscription(subscription_id, destination,
                          event_types, context)
    except ValidationError:
        error_message = "Invalid EventType. The EventTypes are " \
                        "StatusChange, ResourceUpdated, ResourceAdded, " \
                        "ResourceRemoved and Alert."
        abort(status.HTTP_400_BAD_REQUEST, error_message)

    lock = _get_file_lock()
    lock.acquire()
    try:
        # checking for duplicate subscription again to avoid concurrency issue
        is_duplicate = _is_duplicate_subscription(destination)
        if not is_duplicate:
            # write subscription to the all_subscription.json file
            _add_subscription_to_file(sc.redfish)
            for event_type in sc.get_event_types():
                util.get_subscriptions_by_type(
                    )[event_type][subscription_id] = sc

            util.get_all_subscriptions()[subscription_id] = sc
    except Exception as e:
        logging.exception("Error while adding subscription: " + str(e))
        abort(status.HTTP_500_INTERNAL_SERVER_ERROR,
              "Error while adding subscription to the file")
    finally:
        lock.release()
    # Build redfish json
    json_str = sc.serialize()

    # Build response and returns
    response = Response(
        response=json_str,
        status=status.HTTP_201_CREATED,
        mimetype="application/json")
    response.headers.add(
        "Location", "/redfish/v1/EventService/EventSubscriptions/"
                    "{}".format(subscription_id))
    return response


@subscription.route(
    "/redfish/v1/EventService/EventSubscriptions/<subscription_id>",
    methods=["DELETE"])
def remove_subscription(subscription_id):
    """Removes a specific Subscription

        Args:
            subscription_id: The Subscription ID.
    """
    lock = _get_file_lock()
    lock.acquire()
    try:
        sc = util.get_all_subscriptions()[subscription_id]
        event_types = sc.get_event_types()

        # delete subscription from all_subscription file
        _delete_subscription_from_file(subscription_id)

        for event in event_types:
            del util.get_subscriptions_by_type()[event][subscription_id]

        del util.get_all_subscriptions()[subscription_id]

        return Response(
            status=status.HTTP_200_OK,
            mimetype="application/json")
    except KeyError as e:
        logging.exception("Subscription not found: " + str(e))
        abort(status.HTTP_404_NOT_FOUND)
    except Exception as e:
        logging.exception("Error while deleting subscription: " + str(e))
        abort(status.HTTP_500_INTERNAL_SERVER_ERROR)
    finally:
        lock.release()


@subscription.route(
    "/redfish/v1/EventService/EventSubscriptions/<subscription_id>",
    methods=["GET"])
def get_subscription(subscription_id):
    """Gets a specific Subscription

        Args:
            subscription_id: The Subscription ID.

        Returns:
            Subscription JSON.
    """
    try:
        sc = util.get_all_subscriptions()[subscription_id]

        json_str = sc.serialize()
        return Response(
            response=json_str,
            status=status.HTTP_200_OK,
            mimetype="application/json")
    except KeyError as e:
        logging.exception("Subscription not found: " + str(e))
        abort(status.HTTP_404_NOT_FOUND)


def add_subscription_from_file():
    """Add the Redfish Subscription From File.

        EventTypes:
            - ResourceUpdated
            - ResourceAdded
            - ResourceRemoved
            - Alert

        Returns:
            True: If the read from file is successful

    """
    data = get_file_content()
    if data is None:
        return False

    subscription_data = data["members"]
    if not subscription_data:
        return False
    for single_subscription in subscription_data:
        subscription_id = single_subscription["Id"]
        destination = single_subscription["Destination"]
        event_types = single_subscription["EventTypes"]
        context = single_subscription["Context"]

        sc = Subscription(subscription_id, destination,
                          event_types, context)

        for event_type in sc.get_event_types():
            util.get_subscriptions_by_type()[event_type][subscription_id] = sc

        util.get_all_subscriptions()[subscription_id] = sc

    return True



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
import logging
import os
import pika
import ssl
from threading import Thread

# 3rd party libs
from hpOneView.exceptions import HPOneViewException
from hpOneView.resources.resource import ResourceClient
from pika.credentials import ExternalCredentials

# Own libs
from oneview_redfish_toolkit.api.errors import NOT_FOUND_ONEVIEW_ERRORS
from oneview_redfish_toolkit.api.errors import OneViewRedfishException
from oneview_redfish_toolkit.api.event import Event
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import util

SCMB_DIR_NAME = "scmb"
ONEVIEW_CA_NAME = "oneview_ca.pem"
SCMB_CERT_NAME = "oneview_scmb.pem"
SCMB_KEY_NAME = "oneview_scmb.key"
SCMB_PORT = 5671
SCMB_SOCKET_TIMEOUT = 5  # seconds
SCMB_RESOURCE_LIST = [
    'alerts',
    'enclosures',
    'racks',
    'server-hardware']
SCMB_EXCHANGE_NAME = 'scmb'


def init_map_scmb_connections():
    globals()['map_scmb_connections'] = []


def _get_map_scmb_connections():
    return globals()['map_scmb_connections']


def _set_map_scmb_connections(ov_ip):
    globals()['map_scmb_connections'].append(ov_ip)


def _scmb_base_dir():
    certs_dir = os.path.dirname(config.get_config()['ssl']['SSLCertFile'])
    return os.path.join(certs_dir, SCMB_DIR_NAME)


def _scmb_oneview_dir(ov_ip):
    return os.path.join(_scmb_base_dir(), ov_ip)


def _oneview_ca_path(ov_ip):
    return os.path.join(_scmb_oneview_dir(ov_ip), ONEVIEW_CA_NAME)


def _scmb_cert_path(ov_ip):
    return os.path.join(_scmb_oneview_dir(ov_ip), SCMB_CERT_NAME)


def _scmb_key_path(ov_ip):
    return os.path.join(_scmb_oneview_dir(ov_ip), SCMB_KEY_NAME)


def init_event_service(token=None):
    # Iterate through each ov and fork a new thread to listen scmb
    list_ov_ips = config.get_oneview_multiple_ips()

    for ov_ip in list_ov_ips:
        # Loading scmb connection
        # If scmb connection is already present for respective ov then
        # do not fork new thread to listen scmb
        if ov_ip not in _get_map_scmb_connections():
            scmb_thread = SCMB(ov_ip, config.get_credentials(), token)
            scmb_thread.daemon = True
            scmb_thread.start()


class SCMB(Thread):
    def __init__(self, ov_ip, cred, token):
        Thread.__init__(self)
        self.ov_ip = ov_ip
        self.cred = cred
        self.token = token

    def run(self):
        if self._has_valid_certificates():
            logging.info('SCMB certs already exist and are valid...')
        else:
            logging.info('SCMB certs not found. '
                         'Checking if already generated in Oneview...')
            self.get_scmb_certs()
            logging.info('Got certs. Testing connection...')
            if not self._is_cert_working_with_scmb():
                logging.error('Failed to connect to scmb. Aborting...')
                exit(1)

        self._listen_scmb()

    def _has_valid_certificates(self):
        try:
            return self._has_scmb_certificates_path() and \
                self._is_cert_working_with_scmb()
        except KeyError as error:
            logging.error("Invalid configuration for ssl cert. "
                          "Verify the [ssl] section in config file")
            raise error

    def _has_scmb_certificates_path(self):
        return os.path.isfile(_oneview_ca_path(self.ov_ip)) and \
            os.path.isfile(_scmb_cert_path(self.ov_ip)) and \
            os.path.isfile(_scmb_key_path(self.ov_ip))

    def _get_ov_ca_cert(self, ov_client):
        URI = '/rest/certificates/ca'
        resource_client = ResourceClient(ov_client.connection, URI)
        cert = resource_client.get(URI + "?filter=certType:INTERNAL")
        return cert

    def _get_ov_ca_cert_base64data(self, ov_client):
        cert = self._get_ov_ca_cert(ov_client)
        returnCert = None
        if isinstance(cert, dict):
            if 'members' in cert.keys():
                for certObj in cert.get('members'):
                    if certObj and certObj.get('certificateDetails') and \
                            certObj.get('certificateDetails'). \
                            get('base64Data'):
                        returnCert = certObj.get('certificateDetails').\
                            get('base64Data')
                        break
            return returnCert
        # If cert is not a dictionary then returning None
        return returnCert

    def get_scmb_certs(self):
        # Get CA
        ov_client = client_session.get_oneview_client(self.ov_ip, self.token)

        cert = self._get_ov_ca_cert_base64data(ov_client)

        if cert is None:
            raise OneViewRedfishException(
                "Failed to fetch OneView CA Certificate"
            )

        # Create the base scmb dir to save the scmb files
        os.makedirs(name=_scmb_base_dir(), exist_ok=True)

        # Create the dir to save the scmb files for respective ov
        os.makedirs(name=_scmb_oneview_dir(self.ov_ip), exist_ok=True)

        with open(_oneview_ca_path(self.ov_ip), 'w+') as f:
            f.write(cert)

        certs = self._get_scmb_certificate_from_ov(ov_client)

        # Save cert
        with open(_scmb_cert_path(self.ov_ip), 'w+') as f:
            f.write(certs['base64SSLCertData'])

        # Save key
        with open(_scmb_key_path(self.ov_ip), 'w+') as f:
            f.write(certs['base64SSLKeyData'])

    def _generate_certificate_in_oneview(self, ov_client):
        try:
            cert_info = {
                "commonName": "default",
                "type": "RabbitMqClientCertV2"
            }
            ov_client.certificate_rabbitmq.generate(cert_info)
        except HPOneViewException as e:
            # Cert with that commonName already exists. We are going to get it
            if e.oneview_response["errorCode"] == 'RABBITMQ_CLIENTCERT_' \
                                                  'CONFLICT':
                logging.info('Certs already exists in oneview')
            else:
                # Another error is not expected, we raise.
                logging.exception("Unexpected error")
                raise e

    def _get_scmb_certificate_from_ov(self, ov_client):
        cert = None
        try:
            cert = ov_client.certificate_rabbitmq.get_key_pair('default')
            logging.info('SCMB certs retrieved from OneView.')
        except HPOneViewException as e:
            if e.oneview_response["errorCode"] in NOT_FOUND_ONEVIEW_ERRORS:
                logging.info('Generating SCMB cert in Oneview...')
                self._generate_certificate_in_oneview(ov_client)
                cert = ov_client.certificate_rabbitmq.get_key_pair('default')
            else:
                logging.exception("Unexpected error")
                raise e
        return cert

    def scmb_connect(self):
        scmb_server = self.ov_ip

        # Setup our ssl options
        ssl_options = ({'ca_certs': _oneview_ca_path(self.ov_ip),
                        'certfile': _scmb_cert_path(self.ov_ip),
                        'keyfile': _scmb_key_path(self.ov_ip),
                        'cert_reqs': ssl.CERT_REQUIRED,
                        'server_side': False})

        scmb_connection = pika.BlockingConnection(
            pika.ConnectionParameters(
                scmb_server,
                SCMB_PORT,
                credentials=ExternalCredentials(),
                socket_timeout=SCMB_SOCKET_TIMEOUT,
                ssl=True,
                ssl_options=ssl_options))

        return scmb_connection

    def _is_cert_working_with_scmb(self):
        # Create and bind to queue
        EXCHANGE_NAME = 'scmb'
        ROUTE = 'scmb.alerts.#'
        try:
            scmb_conn = self.scmb_connect()
            channel = scmb_conn.channel()
            queue = channel.queue_declare(auto_delete=True)
            channel.queue_bind(
                queue=queue.method.queue,
                exchange=EXCHANGE_NAME,
                routing_key=ROUTE)
            channel.close()
            scmb_conn.close()
        except Exception:
            logging.exception("Failed to test scmb connection")
            return False
        return True

    def consume_message(self, ch, method, properties, body):
        body = json.loads(body.decode('utf-8'))
        resource = body['resource']

        if (resource['category'] == 'alerts'):
            category = resource['associatedResource']['resourceCategory']
        else:
            category = resource['category']

        if (category in SCMB_RESOURCE_LIST):
            event = Event(body)

            util.dispatch_event(event)
        else:
            logging.debug('SCMB message received for an unmanaged resource')

    def _listen_scmb(self):
        try:
            scmb_conn = self.scmb_connect()
            ch = scmb_conn.channel()

            queue_name = ch.queue_declare(auto_delete=True)

            for resource in SCMB_RESOURCE_LIST:
                # scmb.<resource>.#
                route = SCMB_EXCHANGE_NAME + '.' + resource + '.#'

                ch.queue_bind(
                    queue=queue_name.method.queue,
                    exchange=SCMB_EXCHANGE_NAME,
                    routing_key=route)

            ch.basic_consume(self.consume_message,
                             queue=queue_name.method.queue)
            ch.start_consuming()

            # Make scmb enabled flag true for current ov
            _set_map_scmb_connections(self.ov_ip)
        except KeyboardInterrupt:
            ch.close()
            scmb_conn.close()
        except Exception:
            logging.exception("Failed to listen to scmb messages")



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g
from flask import Response
from flask_api import status

# own libs
from hpOneView.exceptions import HPOneViewException
from oneview_redfish_toolkit.api.thermal import Thermal
from oneview_redfish_toolkit import category_resource


thermal = Blueprint("thermal", __name__)


@thermal.route("/redfish/v1/Chassis/<uuid>/Thermal", methods=["GET"])
def get_thermal(uuid):
    """Get the Redfish Thermal for a given UUID.

        Return Thermal Redfish JSON for a given hardware UUID.
        Logs exception of any error and return abort(500)
        Internal Server Error.

        Returns:
            JSON: Redfish json with Thermal
            When hardware is not found calls abort(404)

        Exceptions:
            Logs the exception and call abort(500)

    """
    try:
        category = ''
        cached_category = category_resource.get_category_by_resource_id(uuid)

        if cached_category:
            category = cached_category.resource.replace('_', '-')
        else:
            index_obj = g.oneview_client.index_resources.get_all(
                filter='uuid=' + uuid
            )

            if index_obj:
                category = index_obj[0]["category"]
            else:
                abort(status.HTTP_404_NOT_FOUND, 'Cannot find Index resource')

        if category == 'server-hardware':
            try:
                server_hardware = g.oneview_client.server_hardware. \
                    get_utilization(uuid, fields='AmbientTemperature')
            except HPOneViewException as e:
                abort(status.HTTP_404_NOT_FOUND, e.msg)
            thrml = Thermal(server_hardware, uuid, 'Blade')
        elif category == 'enclosures':
            try:
                enclosure = g.oneview_client.enclosures. \
                    get_utilization(uuid, fields='AmbientTemperature')
            except HPOneViewException as e:
                abort(status.HTTP_404_NOT_FOUND, e.msg)
            thrml = Thermal(enclosure, uuid, 'Enclosure')
        elif category == 'racks':
            rack = g.oneview_client.racks.\
                get_device_topology(uuid)
            thrml = Thermal(rack, uuid, 'Rack')
        else:
            abort(status.HTTP_404_NOT_FOUND, 'OneView resource not found')

        json_str = thrml.serialize()

        return Response(
            response=json_str,
            status=status.HTTP_200_OK,
            mimetype="application/json")
    except HPOneViewException as e:
        # In case of error print exception and abort
        logging.exception(e)

        if e.oneview_response['errorCode'] == "RESOURCE_NOT_FOUND":
            abort(status.HTTP_404_NOT_FOUND, "Resource not found")
        else:
            abort(status.HTTP_500_INTERNAL_SERVER_ERROR)



# -*- coding: utf-8 -*-

# Copyright (2017) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint
from flask import g

from oneview_redfish_toolkit.api.network_adapter_collection \
    import NetworkAdapterCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder

network_adapter_collection = Blueprint(
    "network_adapter_collection", __name__)


@network_adapter_collection.route(
    "/redfish/v1/Chassis/<uuid>/NetworkAdapters/", methods=["GET"])
def get_network_adapter_collection(uuid):
    """Get the Redfish Network Adapters Collection.

    Return NetworkAdapterCollection Redfish JSON.
    """

    server_hardware = g.oneview_client.server_hardware.get_by_id(uuid).data

    nic = NetworkAdapterCollection(server_hardware)

    return ResponseBuilder.success(nic)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask_api import status

# Modules own libs
from oneview_redfish_toolkit import strategy_multiple_oneview as st


RESOURCE_STRATEGY = {
    "appliance_node_information": {
        "get_version": st.all_oneviews_resource,
        "get_status": st.all_oneviews_resource
    },
    "connection": {
        "get": st.first_parameter_resource,
        "post": st.create_server_profile,
        },
    "drive_enclosures": {
        "get": st.first_parameter_resource,
        "get_all": st.drive_enclosures_get_all_with_filter,
    },
    "enclosures": {
        "get": st.first_parameter_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource,
        "get_all": st.all_oneviews_resource,
        "get_environmental_configuration": st.multiple_parameter_resource,
        "get_utilization": st.multiple_parameter_resource,
        },
    "ethernet_networks": {"get": st.first_parameter_resource,
                          "get_by_id": st.first_parameter_resource
                          },
    "index_resources": {
        "get": st.first_parameter_resource,
        "get_all": st.filter_uuid_parameter_resource,
        },
    "logical_enclosures": {
        "get": st.first_parameter_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource,
        "get_all": st.all_oneviews_resource,
        },
    "network_sets": {"get": st.first_parameter_resource,
                     "get_by_id": st.first_parameter_resource,
                     "get_by_uri": st.first_parameter_resource,
                     },
    "racks": {
        "get": st.first_parameter_resource,
        "get_all": st.all_oneviews_resource,
        "get_device_topology": st.first_parameter_resource,
        },
    "sas_logical_jbods": {
        "get": st.first_parameter_resource,
        "get_drives": st.first_parameter_resource,
        },
    "server_hardware": {
        "get": st.first_parameter_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource,
        "get_all": st.all_oneviews_resource,
        "get_utilization": st.multiple_parameter_resource,
        "update_power_state": st.update_power_state_server_hardware,
        },
    "server_hardware_types": {"get": st.first_parameter_resource,
                              "get_by_id": st.first_parameter_resource,
                              "get_by_uri": st.first_parameter_resource
                              },
    "server_profiles": {
        "delete": st.multiple_parameter_resource,
        "get": st.first_parameter_resource,
        "get_all": st.all_oneviews_resource,
        "get_available_targets": st.first_parameter_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource,
        },
    "server_profile_templates": {
        "get": st.first_parameter_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource,
        "get_all": st.spt_get_all_with_filter,
        },
    "tasks": {
        "get": st.first_parameter_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource,
        },
    "labels": {
        "create": st.create_labels,
        "get_by_resource": st.first_parameter_resource
        },
    "sas_logical_interconnects": {
        "get": st.first_parameter_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource
    },
    "volumes": {
        "get_all": st.all_oneviews_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource,
        "get": st.first_parameter_resource,
        },
    "storage_volume_attachments": {
        "get_all": st.all_oneviews_resource,
        },
    "storage_pools": {
        "get": st.first_parameter_resource,
        "get_by_id": st.first_parameter_resource,
        "get_by_uri": st.first_parameter_resource
    }
}


class MultipleOneViewResource(object):

    def __getattribute__(self, name):
        return MultipleOneViewResourceFunction(name)


class MultipleOneViewResourceFunction(object):
    def __init__(self, resource_name):
        self.multiple_ov_resource_name = resource_name

    def __getattribute__(self, name):
        if name == 'multiple_ov_resource_name':
            return object.__getattribute__(self, name)

        return MultipleOneViewResourceRetriever(
            self.multiple_ov_resource_name, name).retrieve


class MultipleOneViewResourceRetriever(object):

    def __init__(self, resource_name, function_name):
        self.multiple_ov_resource_name = resource_name
        self.multiple_ov_function_name = function_name

    def retrieve(self, *args, **kwargs):
        resource = self.multiple_ov_resource_name
        function = self.multiple_ov_function_name

        try:
            get_ov_client_strategy = RESOURCE_STRATEGY[resource][function]
        except KeyError as e:
            msg = 'Not mapped strategy on multiple OneViews' \
                  'support for method {}.{} : {}'. \
                  format(resource, function, e)
            logging.exception(msg)
            abort(status.HTTP_400_BAD_REQUEST, msg)

        result = get_ov_client_strategy(resource, function, *args, **kwargs)

        return result



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class ManagerCollection(RedfishJsonValidator):
    """Creates a Manager Collection Redfish dict

        Populates self.redfish with some hardcoded ManagerCollection
        values and with the response of Oneview with all server hardware,
        enclosures resource.
    """

    SCHEMA_NAME = 'ManagerCollection'

    def __init__(self, oneview_appliances):
        """ManagerCollection constructor

            Populates self.redfish with a hardcoded ManagerCollection
            values and with the response of Oneview with all server hardware,
            enclosures resource.

            Args:
                oneview_appliances: A list of all Oneview's appliances.
        """

        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Manager Collection"
        self.redfish["Members@odata.count"] = len(oneview_appliances)
        self.redfish["Members"] = list()
        self._set_resource_links(oneview_appliances)
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ManagerCollection.ManagerCollection"
        self.redfish["@odata.id"] = "/redfish/v1/Managers"

        self._validate()

    def _set_resource_links(self, oneview_appliances):
        """Populates self.redfish["Members"] with the links resources"""

        for _, appliance_uuid in oneview_appliances.items():
            link_dict = collections.OrderedDict()
            link_dict["@odata.id"] = \
                "/redfish/v1/Managers/" + appliance_uuid
            self.redfish["Members"].append(link_dict)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class Thermal(RedfishJsonValidator):
    """Creates a Thermal Redfish dict

        Populates self.redfish with Thermal data retrieved from OneView
    """

    SCHEMA_NAME = 'Thermal'

    def __init__(self, utilization, uuid, name):
        """Thermal constructor

            Populates self.redfish with the contents of utilization or
            topology dict.

            Args:
                utilization: Hardware utilization or topology dict from OneView
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = "1"
        self.redfish["Name"] = name + " Thermal"
        self.redfish["Temperatures"] = list()
        self.redfish["Temperatures"].append(collections.OrderedDict())
        self.redfish["Temperatures"][0]["MemberId"] = "1"
        self.redfish["Temperatures"][0]["Name"] = "AmbientTemperature"
        self.redfish["Temperatures"][0]["Status"] = collections.OrderedDict()
        self.redfish["Temperatures"][0]["Status"]["State"] = "Enabled"
        self.redfish["Temperatures"][0]["Status"]["Health"] = "OK"
        self.redfish["Temperatures"][0]["PhysicalContext"] = "Intake"
        if name is not 'Rack':
            self.redfish["Temperatures"][0]["ReadingCelsius"] = \
                utilization["metricList"][0]["metricSamples"][0][1]
            self.redfish["Temperatures"][0]["UpperThresholdCritical"] = \
                utilization["metricList"][0]["metricCapacity"]
            self.redfish["Temperatures"][0]["MinReadingRangeTemp"] = 10
            self.redfish["Temperatures"][0]["MaxReadingRangeTemp"] = 35
        else:
            self.redfish["Temperatures"][0]["ReadingCelsius"] = \
                utilization["peakTemp"]
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#Thermal.Thermal"
        self.redfish["@odata.id"] = "/redfish/v1/Chassis/" + uuid + "/Thermal"

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from flask_api import status

NOT_FOUND_ONEVIEW_ERRORS = ['RESOURCE_NOT_FOUND', 'ProfileNotFoundException',
                            'DFRM_SAS_LOGICAL_JBOD_NOT_FOUND',
                            'Cert.CERT_KEY_MISSING_ERROR', 'GENERIC_HTTP_404']

AUTH_ONEVIEW_ERRORS = ['AUTHN_AUTH_FAIL',
                       'AUTHN_AUTH_FAIL_LOGINDOMAINNOTFOUND',
                       'AUTHORIZATION',
                       'Session.INVALID']


class OneViewRedfishException(Exception):

    def __init__(self, msg):
        self.msg = msg


class OneViewRedfishInvalidAttributeValueException(OneViewRedfishException):

    def __init__(self, msg):
        self.msg = msg
        self.status_code_error = status.HTTP_400_BAD_REQUEST


class OneViewRedfishInvalidConditionException(OneViewRedfishException):

    def __init__(self, msg):
        self.msg = msg
        self.status_code_error = status.HTTP_403_FORBIDDEN


class OneViewRedfishResourceNotFoundException(OneViewRedfishException):

    def __init__(self, msg):
        self.msg = msg
        self.status_code_error = status.HTTP_404_NOT_FOUND



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from collections import OrderedDict

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class Subscription(RedfishJsonValidator):
    """Creates the Subscription class

        Populates self.redfish with Subscription response.
    """

    SCHEMA_NAME = 'EventDestination'

    def __init__(self, subscription_id, destination, event_types, context):
        """Session constructor

            Populates self.redfish with Subscription response.

            Args:
                subscription_id: Uniquely identifies the resource within
                the collection of like resources.

                destination: The URI of the destination Event Service.

                event_types: This property shall contain the types of events
                that shall be sent to the destination.

                context: A client-supplied string that is stored with the
                event destination subscription.
        """

        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = subscription_id
        self.redfish["Name"] = "EventSubscription " + subscription_id
        self.redfish["Destination"] = destination
        self.redfish["EventTypes"] = self. \
            _remove_duplicated_event_types(event_types)
        self.redfish["Context"] = context
        self.redfish["Protocol"] = "Redfish"
        self.redfish["SubscriptionType"] = "RedfishEvent"
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#EventDestination.EventDestination"
        self.redfish["@odata.id"] = \
            "/redfish/v1/EventService/EventSubscriptions/" + subscription_id

        self._validate()

    def get_id(self):
        """Gets subscription Id"""
        return self.redfish["Id"]

    def get_event_types(self):
        """Gets the list of event types"""
        return self.redfish["EventTypes"]

    def _remove_duplicated_event_types(self, event_types):
        """Duplicated elements in event_types must be removed.

            OrderedDict keeps the order of the elements,
            so it will preserve list definition.
        """
        # One of the fastest and shortest ways to remove
        # duplicates and keeps the order in Python 3.5
        return list(OrderedDict.fromkeys(event_types))



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from collections import OrderedDict

from oneview_redfish_toolkit.api.computer_system \
    import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.zone_collection \
    import ZoneCollection


class ComputerSystemCollection(RedfishJsonValidator):
    """Creates a Computer System Collection Redfish dict

        Populates self.redfish with some hardcoded ComputerSystemCollection
        values and with the response of Oneview with all servers with
        Server Profile applied.
    """

    SCHEMA_NAME = 'ComputerSystemCollection'

    def __init__(self,
                 server_profile_list,
                 server_profile_templates,
                 zone_ids):
        """ComputerSystemCollection constructor

            Populates self.redfish with a hardcoded ComputerSystemCollection
            values and with the response of Oneview with all servers with
            Server Profile applied and all server profile templates.

            Args:
                server_profile_list: A list of dicts of Server Profile applied
                server_profile_templates: A list of dicts of server profile
                    templates
                zone_ids: A list of Zone Ids
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Computer System Collection"
        server_profile_members_list = \
            self._get_server_profile_members_list(server_profile_list)
        self.redfish["Members@odata.count"] = \
            len(server_profile_members_list)
        self.redfish["Members"] = server_profile_members_list

        self._set_collection_capabilities(server_profile_templates, zone_ids)

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ComputerSystemCollection" \
            ".ComputerSystemCollection"
        self.redfish["@odata.id"] = "/redfish/v1/Systems"

        self._validate()

    def _get_server_profile_members_list(self, server_profile_list):
        """Returns a redfish members list with all server profiles applied

            Iterate over the server hardware list, filter the
            UUIDs of server profiles that are applied and mounts
            the member item to be filled on Redfish Members.

            Args:
                server_profile_list: list of Oneview's Server
                Profiles information.

            Returns:
                list: list of computer system members to be filled on
                Redfish Members.
        """
        members = list()
        for server_profile in server_profile_list:
            members.append({
                "@odata.id": "/redfish/v1/Systems/" + server_profile["uuid"]
            })

        return members

    def _set_collection_capabilities(self, server_profile_templates, zone_ids):
        self.capabilities_key = "@Redfish.CollectionCapabilities"
        self.redfish[self.capabilities_key] = dict()
        self.redfish[self.capabilities_key]["@odata.type"] = \
            self.get_odata_type_by_schema('CollectionCapabilities')
        self.redfish[self.capabilities_key]["Capabilities"] = list()

        for server_profile_template in server_profile_templates:
            spt_id = server_profile_template["uri"].split("/")[-1]

            capability = self._get_capability_object(spt_id, zone_ids)

            self.redfish[self.capabilities_key]["Capabilities"].\
                append(capability)

    def _get_capability_object(self, spt_id, zone_ids):
        capability = OrderedDict()
        capability["CapabilitiesObject"] = dict()
        capability["CapabilitiesObject"]["@odata.id"] = \
            ComputerSystem.BASE_URI + "/" + spt_id
        capability["UseCase"] = "ComputerSystemComposition"
        capability["Links"] = dict()
        capability["Links"]["TargetCollection"] = dict()
        capability["Links"]["TargetCollection"]["@odata.id"] = \
            ComputerSystem.BASE_URI

        zone_ids_filtered = filter(lambda zone_id: spt_id in zone_id, zone_ids)
        self._build_capability_related_items(capability, zone_ids_filtered)

        return capability

    def _build_capability_related_items(self,
                                        capability,
                                        zone_ids_of_capability):
        capability["Links"]["RelatedItem"] = list()
        for zone_id in zone_ids_of_capability:
            related_item = dict()
            related_item["@odata.id"] = ZoneCollection.BASE_URI + "/" + zone_id
            capability["Links"]["RelatedItem"].append(related_item)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from collections import namedtuple

from flask import Response
from flask_api import status

from oneview_redfish_toolkit.api.errors import AUTH_ONEVIEW_ERRORS
from oneview_redfish_toolkit.api.errors import NOT_FOUND_ONEVIEW_ERRORS
from oneview_redfish_toolkit.api.redfish_error import RedfishError


ErrorDescription = namedtuple('ErrorDescription', ['description'])


class ResponseBuilder(object):

    @staticmethod
    def response(serializable_data, http_status, headers={}):
        return Response(
            response=serializable_data.serialize(),
            status=http_status,
            mimetype="application/json",
            headers=headers)

    @staticmethod
    def success(api_data, headers={}):
        return ResponseBuilder.response(api_data, status.HTTP_200_OK, headers)

    @staticmethod
    def success_201(api_data, headers={}):
        return ResponseBuilder.response(api_data, status.HTTP_201_CREATED,
                                        headers)

    @staticmethod
    def error_by_hp_oneview_exception(exception):
        error_code = exception.oneview_response['errorCode']
        http_error_code = status.HTTP_500_INTERNAL_SERVER_ERROR

        if error_code in NOT_FOUND_ONEVIEW_ERRORS:
            http_error_code = status.HTTP_404_NOT_FOUND
        elif error_code in AUTH_ONEVIEW_ERRORS:
            http_error_code = status.HTTP_401_UNAUTHORIZED

        method_name = 'error_' + str(http_error_code)
        handler_method_to_call = getattr(ResponseBuilder, method_name)

        error_desc = ErrorDescription(description=exception.msg)
        return handler_method_to_call(error_desc)

    @staticmethod
    def oneview_redfish_exception(exception):
        method_name = 'error_' + str(exception.status_code_error)
        handler_method_to_call = getattr(ResponseBuilder, method_name)

        error_desc = ErrorDescription(description=exception.msg)
        return handler_method_to_call(error_desc)

    @staticmethod
    def error_401(error):
        redfish_error = RedfishError("GeneralError", error.description)
        return ResponseBuilder.response(redfish_error,
                                        status.HTTP_401_UNAUTHORIZED)

    @staticmethod
    def error_403(error):
        redfish_error = RedfishError("GeneralError", error.description)
        return ResponseBuilder.response(redfish_error,
                                        status.HTTP_403_FORBIDDEN)

    @staticmethod
    def error_404(error):
        redfish_error = RedfishError("GeneralError", error.description)
        return ResponseBuilder.response(redfish_error,
                                        status.HTTP_404_NOT_FOUND)

    @staticmethod
    def error_500(error):
        if hasattr(error, "description"):
            msg = error.description
        else:
            msg = str(error)

        redfish_error = RedfishError("InternalError", msg)
        redfish_error.add_extended_info("InternalError")
        return ResponseBuilder.response(redfish_error,
                                        status.HTTP_500_INTERNAL_SERVER_ERROR)

    @staticmethod
    def error_501(error):
        redfish_error = RedfishError(
            "ActionNotSupported", error.description)
        redfish_error.add_extended_info(
            message_id="ActionNotSupported",
            message_args=["action"])

        return ResponseBuilder.response(redfish_error,
                                        status.HTTP_501_NOT_IMPLEMENTED)

    @staticmethod
    def error_400(error):
        redfish_error = RedfishError(
            "PropertyValueNotInList", error.description)
        return ResponseBuilder.response(redfish_error,
                                        status.HTTP_400_BAD_REQUEST)
